module niprover.

% This module contains a specification of a theorem prover for the I
% natural deduction proof system for first-order intuitionistic logic
% as in Prawitz 1965.

import  nprf.

type    has_proof     nprf -> form -> o.

has_proof (and_i P1 P2) (A and B) :- has_proof P1 A, has_proof P2 B.
has_proof (or_i1 P) (A or B) :- has_proof P A.
has_proof (or_i2 P) (A or B) :- has_proof P B.
has_proof (imp_i P) (A imp B) :-
  pi PA\ ((has_proof PA A) => (has_proof (P PA) B)).
has_proof (neg_i P) (neg A) :-
  pi PA\ ((has_proof PA A) => (has_proof (P PA) false)).
has_proof (exists_i T P) (exists A) :- has_proof P (A T).
has_proof (forall_i P) (forall A) :- pi Y\ (has_proof (P Y) (A Y)).
has_proof (false_i P) A :- has_proof P false.
has_proof (and_e1 B P) A :- has_proof P (A and B).
has_proof (and_e2 A P) B :- has_proof P (A and B).
has_proof (or_e A B P P1 P2) C :- has_proof P (A or B),
  pi PA\ ((has_proof PA A) => (has_proof (P1 PA) C)),
  pi PB\ ((has_proof PB B) => (has_proof (P2 PB) C)).
has_proof (imp_e A P1 P2) B :- has_proof P1 A, has_proof P2 (A imp B).
has_proof (neg_e A P1 P2) false :- has_proof P1 A, has_proof P2 (neg A).
has_proof (exists_e A P1 P2) B :- has_proof P1 (exists A),
  pi Y\ (pi P\ ((has_proof P (A Y)) => (has_proof (P2 Y P) B))).
has_proof (forall_e T A P) (A T) :- has_proof P (forall A).
